#include "occview.h"

#include <Graphic3d_GraphicDriver.hxx>
#include <Xw_Window.hxx>
#include <V3d_DirectionalLight.hxx>
#include <V3d_AmbientLight.hxx>
#include <AIS_Shape.hxx>
#include <BRepPrimAPI_MakeBox.hxx>

OCCView::OCCView(QWidget *parent)
    : QGLWidget(parent)
{
    static Handle(Graphic3d_GraphicDriver) Viewer_aGraphicDriver;
    if (myGraphicDriver.IsNull()) {
        if (Viewer_aGraphicDriver.IsNull()) {
            Handle(Aspect_DisplayConnection) aDisplayConnection;
            aDisplayConnection = new Aspect_DisplayConnection(qgetenv("DISPLAY").constData());
            Viewer_aGraphicDriver = new OpenGl_GraphicDriver (aDisplayConnection);
        }
        myGraphicDriver = Handle(OpenGl_GraphicDriver)::DownCast(Viewer_aGraphicDriver);
    }

    Handle(V3d_Viewer) aViewer = new V3d_Viewer(myGraphicDriver, TCollection_ExtendedString("Visu3D").ToExtString(), "",
                                                1000, V3d_XposYnegZpos,
                                                Quantity_NOC_GRAY30, V3d_ZBUFFER, V3d_GOURAUD, V3d_WAIT,
                                                true, true, V3d_TEX_NONE);

    myView = aViewer->CreateView();
    AISContext = new AIS_InteractiveContext(aViewer);

    Window aWindowHandle = (Window)winId();
    Handle(Aspect_DisplayConnection) aDispConnection = AISContext->CurrentViewer()->Driver()->GetDisplayConnection();
    Handle(Xw_Window) hWnd = new Xw_Window(aDispConnection, aWindowHandle);

    myView->SetWindow(hWnd);
    if(!hWnd->IsMapped())
        hWnd->Map();

    myView->ZBufferTriedronSetup();
    myView->TriedronDisplay(Aspect_TOTP_LEFT_LOWER, Quantity_NOC_BLACK, 0.1, V3d_ZBUFFER);
    aViewer->SetLightOn(new V3d_DirectionalLight(aViewer, V3d_XnegYnegZneg, Quantity_NOC_WHITE, Standard_True));
    aViewer->SetLightOn(new V3d_AmbientLight(aViewer, Quantity_NOC_WHITE));
    aViewer->SetDefaultBackgroundColor(Quantity_NOC_GRAY);

    Handle (AIS_Shape) aBox = new AIS_Shape (BRepPrimAPI_MakeBox(5, 5, 5).Solid());
    AISContext->SetDisplayMode (AIS_Shaded);
    AISContext->Display(aBox);
    myView->FitAll();
}

void OCCView::paintEvent(QPaintEvent* e)
{
    if (!myView.IsNull())
        myView->Redraw();
}

void OCCView::resizeEvent(QResizeEvent* e)
{
    if (!myView.IsNull())
        myView->MustBeResized();
}
